
/*
 * $Header: /home/cvs/CVSROOT/Hausbeschallung/source/P89_1Wire/1wire.c,v 1.3 2006/01/27 16:01:56 cvs Exp $
 */

#include "1wire.h"
#include "delay.h"
#include "crc.h"

#include <stdio.h>


//------------------------------------------------------------
// Global Variales
//
unsigned char xdata owerrno;

static unsigned char xdata rom[8]; 				// ROM Bit
static unsigned char xdata lastDiscrep = 0; 	// last discrepancy
static unsigned char xdata doneFlag = 0; 		// Done flag

//-------------------------------------------------------------
// Local Variables
//
static 			void __ow_write_bit(unsigned char b);
static unsigned char __ow_read_bit(void);
static 			void __ow_write_byte(unsigned char b);
static unsigned char __ow_read_byte(void);

//-------------------------------------------------------------
// Generic Lowlevel IO Functions
// 
#ifdef OW_USE_GENERIC

void __ow_write_byte(unsigned char b)
{
	unsigned char n=8;

	do {
		__ow_write_bit(b&0x01);

		b = b>>1;
	 	n--;
	} while(n);
}

unsigned char __ow_read_byte(void)
{
	unsigned char b=0;
	unsigned char i;

	for (i=0;i<8;i++) {
		if(__ow_read_bit())
			b |= 0x01 << i;
	}

	return b;
}

unsigned char __ow_read_bit(void)
{
	unsigned char i;

	OW_PORT = 0; 			// pull DQ low to start timeslot
	OW_PORT = 1; 			// then return high

	delayus(TRL);   		// delay 15us from start of timeslot  

  	i=OW_PORT;

	delayus(TRWAIT);

	return i;
}


void __ow_write_bit(unsigned char bitval)
{
	OW_PORT = 0; 			// pull DQ low to start timeslot
	if(bitval==1)
		OW_PORT = 1; 		// return DQ high if write 1

	delayus(TW0L); 			// hold value for remainder of timeslot
	OW_PORT=1;
}

//----------------------------------------------------------------------
//
// Generic 1-Wire Functions
//

unsigned char ow_reset()
{
	unsigned char b;

	OW_PORT	=	0;

	delayus(TRSTL);

	OW_PORT	=	1;

	delayus(TMSP);

	b		=	OW_PORT;
	
	delayus(TRSTH);

	return b;
}

#ifdef OW_USE_ROM_SKIP
unsigned char ow_rom_skip()
{

	if ( ow_reset() ) {
		owerrno=OW_EPRESENCE;
		return 1;
	}

	__ow_write_byte(OW_SKIP_ROM);

   	return 0;
}
#endif



unsigned char ow_rom_match(char *rom)
{
	unsigned char n;

	if ( ow_reset() ) {
		owerrno=OW_EPRESENCE;
		return 1;
	}

	/*
	 * Submit a Rom command ...
	 */
	__ow_write_byte(OW_MATCH_ROM);

	for( n=0; n<8; n++) {
		__ow_write_byte(*rom);
		rom++;
	}

	owerrno=OW_OK;
   	return 0;
}

unsigned char ow_rom_read(unsigned char *rom)
{

	unsigned char n;
	unsigned char crc=0;
//	char *p=&rom;

	/*
	 * First we reset the bus and check, if a device ist presend
	 */

 	if ( ow_reset() ) {
		owerrno=OW_EPRESENCE;
		return 1;
	}

	__ow_write_byte(OW_READ_ROM);

	/*
	 * Read retuernd data and calculate checksum
	 */

	for(n=0; n<8; n++) {
		*rom=__ow_read_byte();
		crc8(&crc, *rom);
		rom++;
	}

	if(crc) {
		owerrno=OW_ECRC;
		return 1;
	}

	owerrno=OW_OK;
	return 0;
}

unsigned char ow_rom_access(char *rom)
{

	if(rom)
		return ow_rom_match(rom);
	
	return ow_rom_read(rom);

}
#endif /* OW_USE_GENERIC */

//----------------------------------------------------------------------------------------
// DS18B20 Device Specific Functions
//
#ifdef OW_USE_DS18B20

unsigned char ow_function_read_sp8(unsigned char *rom, unsigned char *buf, int len)
{
	unsigned char crc=0;
 	
	if (ow_rom_access(rom)) {	   	// Reset Bus and read ROM
		return 1;
	}

	__ow_write_byte(OW_READ_SP);

	while(len--) {
		*buf=__ow_read_byte();
		crc8(&crc, *buf);
	//	printf("%bx ", crc);	
		buf++;
	}
	//putchar('\n');
	if (crc) {
		owerrno=OW_ECRC;
		printf("sp crc error: %bx\n", crc);
		return 1;
	}

	owerrno=OW_OK;
	return 0;
}



/*
 * 	ow_function_write_sp8
 *
 * Write Data to DS18B20 scratchpad
 *
 * *rom    --- pointer device from address
 * *buf    --- pointer to data to be written
 * len     --- length of data in *buf
 *
 */
unsigned char ow_function_write_sp8(unsigned char *rom, unsigned char *buf, int len)
{

	if (ow_rom_access(rom)) {	   	// Reset Bus and read ROM
		return 1;
	}

	__ow_write_byte(OW_WRITE_SP);
	
	while(len--) {
		__ow_write_byte(*buf);
		buf++;
	}

	return 0;
}

/*
 * 	ow_function_copy_sp
 *
 * Copy DS18B20 scratchpad to EEPROM
 *
 * *rom    --- pointer device from address
 *
 */
unsigned char ow_function_copy_sp8(unsigned char *rom)
{
 	
	if (ow_rom_access(rom)) 				// Reset Bus and read ROM
		return 1;

	__ow_write_byte(OW_COPY_SP);


	while( !__ow_read_bit())
		;
	
	SCL=1;
	return 0;
}

/*
 * Start a Temperature Conversion a wait until it is finisched
 */

unsigned char ow_function_convert_temp(unsigned char *rom)
{
 	
	if (ow_rom_access(rom))			 	// Reset Bus and read ROM
		return 1;

	__ow_write_byte(OW_CONVERT_T);

	while( !__ow_read_bit() ) {
		 ;
	}

	return 0;
}

/*
 * Convert Temperature into a signed float number
 */

float ow_normalize_temp(unsigned char *rom)
{
	float 	temp =0;
 	bit 	sign =0;

	if((*(rom+1)&0x80)) { // Negative
		*rom=~ *rom;
		*(rom+1)=~*(rom+1);
		sign=1;
	}
	temp += (*rom &0x0f) / 16.0; // Nachkommazahl
	temp += ((*rom >> 4));
	temp += (*(rom+1) & 0x0f)*16;

	if(sign) { // Negative
	   	temp = temp+1;
		temp = -temp;
	}		

	return temp;

}

#endif /* OW_USE_DS18B20 */

//----------------------------------------------------------------------------------------
// DS28E04 Device Specific Functions
//

#ifdef OW_USE_DS24E04

/*
 * 	ow_function_read_sp16
 *
 * Read Data from DS28E04 scratchpad
 *
 * *rom    --- pointer device from address
 * *buf    --- pointer to receiver buffer. must be large enough to store the whole
 *            scratchpad (32 bytes)
 * *buflen --- length of data, actually received
 * *auth   --- pointer to sturtc where the received authentication token will bestored.
 *
 */

unsigned char ow_function_read_sp16(unsigned char *rom, unsigned char *buf, unsigned char *buflen, ow_write_auth *auth)
{
	unsigned int crc=0;
	unsigned char i;
 	unsigned char a1, a2, es;

	if (ow_rom_access(rom)) {
		return 1;
	}

	__ow_write_byte(OW_READ_SP16);
	crc16(&crc, OW_READ_SP16);

	a1=__ow_read_byte();
	crc16(&crc, a1);
	a2=__ow_read_byte();
	crc16(&crc, a2);
	es=__ow_read_byte();
	crc16(&crc,es);

	if(auth) {
		auth->t1=a1;
		auth->t2=a2;
		auth->es=es;
	}

	for(i=0; i < es - (a1&0x1f)+1;  i++ ) {
		*buf=__ow_read_byte();
		crc16(&crc, *buf);
		buf++;
	}
	if(buflen) 
		*buflen=i;

	crc16(&crc, ~__ow_read_byte());

	crc16(&crc, ~__ow_read_byte());

	if (crc) {
		owerrno=OW_ECRC;
		printf("sp crc error: %bx\n", crc);
		return 1;
	}

	owerrno=OW_OK;
	return 0;
}

/*
 * 	ow_function_write_sp16
 *
 * Write data to DS28E04 scratchpad 
 *
 * *rom    --- pointer device from address
 * page    --- EEPROM page data is written to 0 ... 15
 * *buf    --- pointer to data to be written
 * len     --- length of data in buf to be written
 * offset  --- offset within a page, where the data is stored
 *
 */
unsigned char ow_function_write_sp16(unsigned char *rom, unsigned char page, unsigned char *buf, unsigned char len, unsigned char offset)
{
	unsigned int crc=0;
	unsigned char b=0;
	unsigned char i, t1, t2;

	if ( page > 15 || offset > 32 || (len + offset) > 32) {
		owerrno=OW_ERANGE;
		return 1;
	}

 	if (ow_rom_access(rom))
			return -1;
	
	__ow_write_byte(OW_WRITE_SP16);
	crc16(&crc, OW_WRITE_SP16);

	t1= (page<<5) | offset;
	t2=(page&0x08)>>3;

	__ow_write_byte(t1);	
	crc16(&crc, t1);

	__ow_write_byte(t2);	
	crc16(&crc, t2);

	for(i=0; i<len; i++) {
			__ow_write_byte(*buf);
			crc16(&crc, *buf);
			buf++;
	}
	if ( len == OW_DS28E04_SP_SIZE ) {
		crc16(&crc, ~__ow_read_byte());
		crc16(&crc, ~__ow_read_byte());

		if (crc) {
			owerrno=OW_ECRC;
			return 1;
		}
	}

	owerrno=OW_OK;

	return 0;
}


//----------------------------------------------------------------------------------
//  ow__function_copy_sp16
//
//  Copy DS28E04 scratchpad to EEPROM
//
//	*rom    --- pointer device from address
//	*auth   --- pointer to authentication token
//
unsigned char ow_function_copy_sp16(char *rom, ow_write_auth *auth)
{

	if (ow_rom_access(rom)) 				// Reset Bus and read ROM
		return 1;

	__ow_write_byte(OW_COPY_SP16);
	
	__ow_write_byte(auth->t1);
	__ow_write_byte(auth->t2);
	__ow_write_byte(auth->es);

	delayms(10);

	if(__ow_read_byte() != OW_DS28E04_AUTH_OK) {
		owerrno=OW_EAUTHERR;
		return 1;
	}

	owerrno=OW_OK;
	return 0;

}

//---------------------------------------------------------------------------------
// 	NAME: ow_function_read_memory
//
//  DESCRPTION: Read EEPROM memory from DS24E04 
//
//  PARAMETERs:
//  *rom 		Pointer to Device
//	addr		EEPROM address to be read
// 	*buf		Pointer to return buffer
//	len			Number of bytes to be read
//
//	RETURN:		0 on success, 1 on error
//
unsigned char ow_function_read_memory(unsigned char *rom, unsigned int addr, unsigned char *buf, int len)
{

 	if(ow_rom_access(rom))
		return 1;

	__ow_write_byte(OW_READ_MEMORY);

	__ow_write_byte(addr&0xff);
	__ow_write_byte( (addr&0xff00)>> 8);

	while(len --) {
	 	*buf=__ow_read_byte();
	  	buf++;
	}
	return 0;
}

//---------------------------------------------------------------------------------
// 	ow_function_pio_access_read
//
//  Read PIO Ports from DS24E04 
//

#ifdef OW_USE_DS24E04_PIO

unsigned char ow_function_pio_read(unsigned char *rom, unsigned char *pio)
{
	unsigned char	i;
	unsigned int 	crc=0;

 	if(ow_rom_access(rom))
		return 1;

   	__ow_write_byte(OW_PIO_ACCESS_READ);
	crc16(&crc, OW_PIO_ACCESS_READ);	

	for(i=0; i<32; i++) {
		*pio=__ow_read_byte();
		crc16(&crc, *pio);	
	}

	crc16(&crc, ~__ow_read_byte());
	crc16(&crc, ~__ow_read_byte());		
	
	if (crc) {
		owerrno=OW_ECRC;
		return 1;
	}
   	return 0;
}

unsigned char ow_function_pio_write(unsigned char *rom, unsigned char in, unsigned char *out)
{
	unsigned char r;

	if(ow_rom_access(rom))
		return 1;
	
	in |= 0xfc;  // Only the lower two bits are of interrest, all others should be 1's


   	__ow_write_byte(OW_PIO_ACCESS_WRITE);
	__ow_write_byte(in);
	__ow_write_byte(~in);

	if ( __ow_read_byte() != 0xaa ) {
		printf("PIO WRITE ERROR\n");
		return 1;
	}

	r=__ow_read_byte();

	if ( out ) 
		*out=r;

	owerrno=0;
	return 0;

}
#endif	/* OW_USE_DS24E04_PIO */

#endif /* OW_USE_DS24E04 */

//--------------------------------------------------------------------------------
// 1-Wire Device Search
// Adapted code from Dallas Application Note

#ifdef OW_USE_DEVICE_SEARCH
// 
// The Next function searches for the next device on the 1-wire bus. If
// there are no more devices on the 1-wire then false is returned.
//
static unsigned char __ow_find_next(void)
{
	unsigned char m = 1; // ROM Bit index
	unsigned char n = 0; // ROM Byte index
	unsigned char k = 1; // bit mask
	unsigned char x = 0;
	unsigned char discrepMarker = 0; // discrepancy marker
	unsigned char g; // Output bit
	unsigned char nxt; // return value
	unsigned char dowcrc;
	int flag;

	nxt = 0; // set the next flag to false
	dowcrc = 0; // reset the dowcrc
	flag = ow_reset(); // reset the 1-wire
	if(flag||doneFlag) // no parts -> return false
	{
		lastDiscrep = 0; // reset the search
		return 0;
	}
	__ow_write_byte(OW_SEARCH_ROM); // send SearchROM command
	do
	// for all eight bytes
	{
		x = 0;
		if(__ow_read_bit()==1) x = 2;
		//delayus(6);
		if(__ow_read_bit()==1) x |= 1; // and its complement
		if(x ==3)  { // there are no devices on the 1-wire
			break;
		}
		else
		{
		//	printf("Something\n");
			if(x>0) // all devices coupled have 0 or 1
				g = x>>1; // bit write value for search
			else
			{
				// if this discrepancy is before the last
				// discrepancy on a previous Next then pick
				// the same as last time
				if(m<lastDiscrep)
				g = ((rom[n]&k)>0);
				else // if equal to last pick 1
				g = (m==lastDiscrep); // if not then pick 0
				// if 0 was picked then record
				// position with mask k
				if (g==0) discrepMarker = m;
			}
			if(g==1) // isolate bit in ROM[n] with mask k
				rom[n] |= k;
			else
				rom[n] &= ~k;
		//	printf("n=%bx %bx\n", n, ROM[n]);
			__ow_write_bit(g); // ROM search write
	
			m++; // increment bit counter m
			k = k<<1; // and shift the bit mask k
			if(k==0) // if the mask is 0 then go to new ROM
			{ // byte n and reset mask
				crc8(&dowcrc, rom[n]); // accumulate the CRC
				n++; k++;
			}
		}
	}	while(n<8); //loop until through all ROM bytes 0-7

	if(m<65 ||dowcrc ) // if search was unsuccessful then
		lastDiscrep=0; // reset the last discrepancy to 0
	else
	{
		// search was successful, so set lastDiscrep,
		// lastOne, nxt
		lastDiscrep = discrepMarker;
		doneFlag = (lastDiscrep==0);
		nxt = 1; // indicates search is not complete yet, more
		// parts remain
	}
	return nxt;
}


// FIRST
// The First function resets the current state of a ROM search and calls
// Next to find the first device on the 1-wire bus.
//
static unsigned char __ow_find_first(void)
{
	lastDiscrep = 0; // reset the rom search last discrepancy global
	doneFlag = 0;
	return __ow_find_next(); // call Next and return its return value
}


/****************************************************************************
 * ow_find_devices -- Find up to ndevice 1-Wire devices and
 *                    place rom codes in roms
 *					  Return number of found devices
 ***************************************************************************/

unsigned char ow_find_devices(char *roms, unsigned char ndevices)
{
	unsigned char i;
	char *p;
	
	p=roms;
 
	if(!ow_reset()) //Begins when a presence is detected
	{
		if(__ow_find_first()) //Begins when at least one part is found
		{
		
			do{
				for(i=0;  i<8; i++) {
					*p=rom[i]; //Identifies ROM number on found device
					p++;
				}
			}while (__ow_find_next()&&(--ndevices)); //Continues until no additional devices are found
		}
	}
	return (p-roms)/8;
}

unsigned char *ow_has_device(unsigned char *roms, unsigned char ndevices, unsigned char devcode )
{
	unsigned char dev=0;

	while(ndevices--) {
		dev++;
		if ( *roms == devcode ) {
			return roms;
		}
		roms+=8;
	}

	return NULL;

}


void ow_print_devices(char *roms, unsigned char ndevices)
{
	unsigned char i;

	if (!ndevices) {
		printf("No 1-Wire devices found\n");
		return;
	}

	printf("\nFound %bd 1-Wire devices\n\n", ndevices); 
	
	while(ndevices--) {
		switch(*roms) {
			case OW_DS18B20:
				printf("DS18B20 : ");
				break;
			case OW_DS28E04:
				printf("DS28E04 : ");
				break;
			case OW_DS2404:
				printf("DS2404  : ");
				break;
			default:
				printf("Unknown : ");
				break;
			}
		for(i=0; i<8; i++) {
			printf("%2bx ", *roms);
			roms++;
		}
		putchar('\n');
	}
	putchar('\n');		
}

#endif /* OW_USE_DEVICE_SEARCH */
