

// $Header: /home/cvs/CVSROOT/Hausbeschallung/source/P89_1Wire/1wire.h,v 1.2 2006/01/08 16:43:09 cvs Exp $

#ifndef OWIRE_H
#define OWIRE_H

//----------------------------------------
// User Configurable Driver parameters

#define SIXCLOCK
#define OW_USE_GENERIC
#define OW_USE_DS18B20
#define OW_USE_DS24E04
// #define OW_USE_DS24E04_PIO
#define OW_USE_DEVICE_SEARCH

//-----------------------------------------------------------------
// P89C668 specific timings, 6CLK Mode
//

#ifdef SIXCLOCK

#include <reg668.h>

#define OW_PORT SDA

#define TRSTL 147 // 480us
#define TRSTH 130 // 424us
#define TMSP   20 // 70us 

// Write one
#define TW1L    1
#define TW1WAIT  16	 // 59us

// Write zero
#define TW0L     30	 // 100us
#define TW0WAIT  1	 //  10us

// Read bit
#define TRL		 3   // 16us
#define TRWAIT  35	 // 120us

#else
//------------------------------------------------
// 8051 standard mode, 12CLK timing
//

#include <reg5000.h>

sbit OW_PORT  	= P1^7;
sbit SCL  		= P1^6;

#define TRSTL 73 	// 480us
#define TRSTH 64 	// 424us
#define TMSP   9 	// 70us 

// Write one
#define TW1L    1	// TODO
#define TW1WAIT  7 // 59us

// Write zero
#define TW0L     13		// 100us
#define TW0WAIT  1	 //  10us TODO

// Read bit
#define TRL		 1   		// 16us
#define TRWAIT  16	 	// 120us

#endif /* SICKCLOCK */


// 1-Wire Device Identifiers
#define OW_DS18B20			0x28 	// Thermometer
#define OW_DS28E04			0x1c 	// 4096 bit EERPOM
#define OW_DS2404			0x04	// EconoRAM Time Chip

// OW ROM Commands
#define OW_SEARCH_ROM 		0xF0
#define OW_READ_ROM 		0x33
#define OW_SKIP_ROM			0xcc
#define OW_MATCH_ROM		0x55

// Device specific Commands DS18B20
#define OW_CONVERT_T		0x44
#define OW_READ_SP			0xBE
#define OW_READ_POWER  		 0xB4
#define OW_WRITE_SP  		0x4e
#define OW_COPY_SP			0x48

// Device Specific Commands DS28E04
#define OW_PIO_ACCESS_READ 	0xf5
#define OW_PIO_ACCESS_WRITE 0x5a
#define OW_READ_MEMORY 		0xf0
#define OW_READ_SP16		0xaa
#define OW_WRITE_SP16		0x0f
#define OW_COPY_SP16		0x55

#define OW_DS28E04_AUTH_OK	0xaa
#define OW_DS28E04_SP_SIZE  0x20

// OW error condition
#define OW_OK				0x00
#define OW_EPRESENCE 		0x01	// Device not present
#define OW_ECRC		 		0x02	// CRC error
#define OW_EAUTHERR			0x03	// Authentication token invalid
#define OW_ERANGE			0x04    // Memory access out of range

// OW Conriguration Register Setting
#define OW_TEMP_RES_9 		0x1f
#define OW_TEMP_RES_10 		0x3f
#define OW_TEMP_RES_11 		0x5f
#define OW_TEMP_RES_12 		0x7f


struct ow_write_auth {
 	unsigned char t1, t2, es;
};
typedef struct ow_write_auth ow_write_auth;


// Global error variable
extern unsigned char xdata owerrno;


// General ROM functions
#ifdef OW_USE_GENERIC
extern unsigned char 	ow_reset(void);
extern unsigned char 	ow_rom_read(unsigned char *rom);
extern unsigned char 	ow_rom_match(char *rom);
extern unsigned char 	ow_rom_access(char *rom);
#ifdef OW_USE_ROM_SKIP
extern unsigned char 	ow_rom_skip(void);
#endif /* OW_USE_ROM_SKIP */
#endif /* OW_USE_GENERIC */

// DS1820 specific functtions
#ifdef OW_USE_DS18B20
extern unsigned char 	ow_function_read_sp8(unsigned char *rom, unsigned char *buf, int len);
extern unsigned char 	ow_function_write_sp8(unsigned char *rom, unsigned char *buf, int len);
extern unsigned char 	ow_function_copy_sp8(unsigned char *rom);
extern unsigned char 	ow_function_convert_temp(unsigned char *rom);
extern float 			ow_normalize_temp(unsigned char *rom);
#endif	/* OW_USE_DS18B20 */

// DS24E04 specific functions
#ifdef OW_USE_DS24E04
extern unsigned char 	ow_function_read_sp16(unsigned char *rom, unsigned char *buf, unsigned char *buflen, ow_write_auth *);
extern unsigned char 	ow_function_write_sp16(unsigned char *rom, unsigned char page, unsigned char *buf, unsigned char len, unsigned char offset);
extern unsigned char 	ow_function_copy_sp16(char *rom, ow_write_auth *auth);
extern unsigned char 	ow_function_read_memory(unsigned char *rom, unsigned int addr, unsigned char *buf, int len); 
#ifdef OW_USE_DS24E04_PIO
extern unsigned char 	ow_function_pio_read(unsigned char *rom, unsigned char *pio);
extern unsigned char 	ow_function_pio_write(unsigned char *rom, unsigned char in, unsigned char *out);
#endif /*  OW_USE_DS24E04_PIO */


#endif	/* OW_USE_DS24E04 */

// Devicee search functions
#ifdef OW_USE_DEVICE_SEARCH
extern unsigned char 	ow_find_devices(char *r, unsigned char);
extern unsigned char 	*ow_has_device(unsigned char *roms, unsigned char ndevices, unsigned char devcode );
extern void 			ow_print_devices(char *roms, unsigned char ndevices);
#endif /*  OW_USE_DEVICE_SEARCH */

#endif /* OWIRE_H */
