#include <reg668.h>
// #include <REG5000.H>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "1wire.h"

#define XTAL 11059200
#define BAUD 57600

#include <float.h>

/* Configure Timer 0 with 1kHz */
#define TIMER0_L  205
#define TIMER0_H  248

void hexdump(char *pre, char *p, unsigned int len)
{
   	unsigned char i=0;
	unsigned int j;

	if(pre)
		puts(pre);

	for(j=0; j<len; j++) {
		if (  i %16 == 0 ) {
			printf("%4X    :  ", j);
		}
		printf("%2bx ", *p++);
		i++;
		if (i%16 == 0)
			putchar('\n');
	}
	putchar('\n');
}

//unsigned char __mgetkey ()  {
//  unsigned char c;
//  
//  if(!RI)  {
//  	return 0;
//}
//
//  c = S0BUF;
//  RI = 0;
//  // putchar(c);
//  return (c);
//}


void main(void)
{ 
   	unsigned int t;
	unsigned char xdata rom[9], *ds18b20, *ds28e04;
	unsigned char xdata buf[0x226];
	float tl=FLT_MAX, th=FLT_MIN;
	unsigned char xdata nr[5][8];
	unsigned char  ndevices, b, page;
	ow_write_auth xdata auth;
//	unsigned char  b;
    S0CON  = 0x50;		        /* SCON: mode 1, 8-bit UART, enable rcvr      */
	PCON &= 0x7f;				/* SMOD == 1 double baud-rate                 */
//	PCON |= 0x80;
	TMOD &= 0xCF;
    TMOD |= 0x20;               /* TMOD: timer 1, mode 2, 8-bit reload        */
	TH1  = (unsigned char) (256-(XTAL / (16L*12L*BAUD)));
	// TH1=0xFD;
    TR1   = 1;                  /* TR1:  timer 1 run                          */
    TI    = 1;                  /* TI:   set TI to send first char of UART    */

	/* Setup timerinterrupt every 10ms */
 	TH0 = TIMER0_H;                						/* set timer0 period            */
  	TL0 = TIMER0_L;
  	TMOD = TMOD | 0x01;                          	/* select mode 2               */
  	TR0 = 0;                                     	/* start timer 0               */
  	ET0 = 0;                                     	/* enable timer 0 interrupt    */
  
  	EA  = 1; 										/* enable all interrrupst      */ 
	
	AUXR=0xfd; 										/* Enable Internal RAM, Disable ALE	 */

	printf("RUNNING!!!\n");

	memset(&nr, 0, sizeof(nr));
	ndevices=ow_find_devices((char *)&nr, sizeof(nr)/8);

	ow_print_devices((char *)&nr, ndevices);

	if ( (ds18b20=ow_has_device((char *)&nr, ndevices, OW_DS18B20)) ) {
		printf("DS18B20 Found\n");
	}
	if ( (ds28e04=ow_has_device((char *)&nr, ndevices, OW_DS28E04)) ) {
		printf("DS28E04 Found\n");
	}
	
	//// Erase Flash
	memset(&buf, 0x00, sizeof(buf));
	for(page=0; page<16; page++) {
		if (ow_function_write_sp16(ds28e04, page, &buf, OW_DS28E04_SP_SIZE, 0)) {
	  		printf("write_sp16 error: %bx\n", owerrno);
		}
	
		if (ow_function_read_sp16(ds28e04, &buf, &b, &auth)) {
 			printf("read sp16 error: %bx\n", owerrno);	
		}

		if (ow_function_copy_sp16(ds28e04, &auth)) {
		   printf("copy sp error: %bx\n", owerrno);	
		}
	putchar('e');
	}
	putchar('\n');

	memset(&buf, 0xbb, sizeof(buf));		   

	if (ow_function_write_sp16(ds28e04, 1, &buf, 5, 3)) {
	  	printf("write_sp16 error: %bx\n", owerrno);
	}
	
	memset(&buf, 0, sizeof(buf));		   

	if (ow_function_read_sp16(ds28e04, &buf, &b, &auth)) {
	 	printf("read sp16 error: %bx\n", owerrno);	
	}
	printf("Read sp: %bx bytes\n", b);
	//hexdump(&buf, 32);

	if (ow_function_copy_sp16(ds28e04, &auth)) {
	   printf("copy sp error: %bx\n", owerrno);	
	}
	

   	if (ow_function_read_memory(ds28e04, 0x0000, &buf, 0x226)) {
	 	printf("read mme error: %bx\n", owerrno);
	}
	hexdump("Memory Dump", &buf, sizeof(buf));

	printf("Running\n");

	hexdump("Rom DS18B20", ds18b20, 8);
	
	rom[0]=0x099;
	rom[1]=0x088;
	rom[2]=OW_TEMP_RES_12;
		
	ow_function_write_sp8(ds18b20, &rom, 3);

//	memset(&rom, 0, sizeof(rom));
//
//	ow_function_read_sp8(ds18b20, &rom, 9);

	ow_function_copy_sp8(ds18b20);

	while(1) {
		unsigned char temp=0;
		static float old=0;
		float temp2=0;

		if (ow_function_convert_temp(ds18b20) || ow_function_read_sp8(ds18b20, &rom, 9)) {
			printf("Error: %bx\n", owerrno);
			continue;
		}

		temp2=ow_normalize_temp(&rom);

		if(temp2!=old) {

			old=temp2;
		}
		if ( old < tl ) {
			tl=old;
			printf("min=%f	max=%f\n", tl, th);
		}
		if ( old > th ) {
			th=old;
			printf("min=%f	max=%f\n", tl, th);
		}
		for(t=0; t<1000; t++);
	}

}


void timer0_ISR (void) interrupt 1 using 1 {   /* Int Vector at 000BH, Reg Bank 1 */

//	static unsigned char div=0;

	TH0 = TIMER0_H;        /* set timer0 period  to 1ms   */
  	TL0 = TIMER0_L;
}
